use std::path::PathBuf;
use async_trait::async_trait;
use j_db::database::Database;
use serde::{Deserialize, Serialize};
use sha2::Digest;
use url::Url;
use crate::model::image::{Image, ImageData, StorageLocation};
use crate::storage_manager::{Store, StoreError};

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct FileStoreConfig {
    pub base_path: PathBuf,
    pub base_url: String,

    pub max_file_size: usize,
    pub max_total_storage: usize,
}

#[derive(Debug, Clone)]
pub struct FileStore {
    pub config: FileStoreConfig
}

impl FileStore {
    pub fn new(config: FileStoreConfig) -> Self {
        Self {
            config
        }
    }
}

#[async_trait]
impl Store for FileStore {
    async fn store_img(&mut self, img_data: ImageData, file_name: &str) -> Result<(Url, StorageLocation), StoreError> {
        let img_data = match img_data {
            ImageData::Bytes(b) => b,
            ImageData::Link(_) => unimplemented!("No link support")
        };
        let hash = sha2::Sha256::digest(&img_data);
        let disk_file_name = hex::encode(hash);

        let file = PathBuf::from(file_name);
        let ext = file.extension().unwrap().to_str().unwrap();

        let disk_file_name = format!("{}.{}", disk_file_name, ext);

        let path = self.config.base_path.join(&disk_file_name);

        tokio::fs::write(&path, img_data).await?;

        let img_link = Url::parse(&self.config.base_url).unwrap().join(&disk_file_name).unwrap();

        let storage_location = StorageLocation::FileStore {path};

        Ok((img_link, storage_location))
    }

    async fn delete_img(&mut self, img: Image) -> StoreError {
        todo!()
    }

    fn max_image_size(&self) -> usize {
        self.config.max_file_size
    }

    fn max_total_storage(&self) -> usize {
        self.config.max_total_storage
    }

    fn current_store_size(&self, _db: &Database) -> usize {
        0
    }
}