use std::fmt::{Display, Formatter};
use axum::async_trait;
use j_db::database::Database;
use serde::{Deserialize, Serialize};
use url::Url;
use crate::model::image::{Image, ImageData, StorageLocation};

pub mod file_store;

#[derive(Debug)]
pub enum StoreError {
    InvalidFile,
    ImageNotFound,
    OutOfStorage,
    ImageTooBig,
    IOError(tokio::io::Error)
}

impl Display for StoreError {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            StoreError::InvalidFile => write!(f, "Invalid file"),
            StoreError::ImageNotFound => write!(f, "Image not found"),
            StoreError::IOError(err) => write!(f, "IO Error: {}", err),
            StoreError::OutOfStorage => write!(f, "Underlying store full"),
            StoreError::ImageTooBig => write!(f, "Image too big for store")
        }
    }
}

impl std::error::Error for StoreError {}

impl From<tokio::io::Error> for StoreError {
    fn from(value: tokio::io::Error) -> Self {
        Self::IOError(value)
    }
}

#[async_trait]
pub trait Store {
    async fn store_img(&mut self, img_data: ImageData, file_name: &str) -> Result<(Url, StorageLocation), StoreError>;

    async fn create_img(&mut self, img_data: ImageData, file_name: &str, created_by: u64) -> Result<Image, StoreError> {

        let (url, storage_location) = self.store_img(img_data, file_name).await?;

        Ok(Image::new(file_name, Vec::new(), url, created_by, storage_location))
    }

    async fn delete_img(&mut self, img: Image) -> StoreError;

    fn max_image_size(&self) -> usize;

    fn max_total_storage(&self) -> usize;

    fn current_store_size(&self, db: &Database) -> usize;
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum StorageTypes {
    FileStore,
    LinkStore
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct StorageManagerConfig {

}

#[derive(Debug, Clone)]
pub struct StorageManager {

}